package oxalis.thirdparty

import com.scalableminds.util.security.SCrypt._
import models.user.User
import play.api.libs.ws.WS
import com.ning.http.client.Realm.AuthScheme
import play.api.libs.concurrent.Execution.Implicits._
import play.api.Logger
import play.api.Play.current
import play.api.Play
import scala.concurrent.Promise
import scala.concurrent.Future
import scala.util._
import play.api.libs.concurrent.Akka
import models.annotation.{AnnotationLike, AnnotationDAO}
import models.tracing.skeleton.SkeletonTracing
import com.scalableminds.util.reactivemongo.{DBAccessContext, GlobalAccessContext}

object BrainTracing {
  val URL = "http://braintracing.org/"
  val CREATE_URL = URL + "oxalis_create_user.php"
  val LOGTIME_URL = URL + "oxalis_add_hours.php"
  val USER = "brain"
  val PW = "trace"
  val LICENSE = "hu39rxpv7m"

  val isActive = Play.configuration.getBoolean("braintracing.active") getOrElse false
  val logTimeForExplorative = Play.configuration.getBoolean("braintracing.logTimeForExplorative") getOrElse false

  def register(user: User, password: String): Future[String] = {
    val pwHash = md5(password)
    // TODO: fix, make team dynamic
    if (isActive && user.teamNames.contains("Structure of Neocortical Circuits Group")) {
      val result = Promise[String]()
      WS
      .url(CREATE_URL)
      .withAuth(USER, PW, AuthScheme.BASIC)
      .withQueryString(
        "license" -> LICENSE,
        "firstname" -> user.firstName,
        "lastname" -> user.lastName,
        "email" -> user.email,
        "pword" -> pwHash)
      .get()
      .map { response =>
        result complete (response.status match {
          case 200 =>
            Success("braintracing.new")
          case 304 =>
            Success("braintracing.exists")
          case _ =>
            Success("braintraceing.error")
        })
        Logger.trace(s"Creation of account ${user.email} returned Status: ${response.status} Body: ${response.body}")
      }
      result.future
    } else {
      Future.successful("braintracing.new")
    }
  }

  private def inHours(millis: Long) =
    millis / (1000.0 * 60 * 60)

  def logTime(user: User, time: Long, annotation: Option[AnnotationLike])(implicit ctx: DBAccessContext): Unit = {
    import scala.async.Async._
    // TODO: fix, make team dynamic
    if (isActive && user.teamNames.contains("Structure of Neocortical Circuits Group")) {
      async {
        val task = await(annotation.toFox.flatMap(_.task).futureBox)
        val taskTypeFox = task.toFox.flatMap(_.taskType)
        val project = task.toFox.flatMap(_.project)
        if (logTimeForExplorative || task.isDefined) {
          val hours = inHours(time)
          val projectName = await(project.map(_.name).getOrElse(""))
          val taskType = await(taskTypeFox.futureBox)
          WS
          .url(LOGTIME_URL)
          .withAuth(USER, PW, AuthScheme.BASIC)
          .withQueryString(
            "license" -> LICENSE,
            "email" -> user.email,
            "hours" -> hours.toString,
            "tasktype_id" -> taskType.map(_.id).getOrElse(""),
            "tasktype_summary" -> taskType.map(_.summary).getOrElse(""),
            "task_id" -> task.map(_.id).getOrElse(""),
            "project_name" -> projectName
          )
          .get()
          .map { response =>
            response.status match {
              case 200 =>
                Logger.trace(s"Logged time! User: ${user.email} Time: $hours")
              case code =>
                Logger.error(s"Time logging failed! Code $code User: ${user.email} Time: $hours")
            }
          }
        }
      }
    }
  }
}